#include "derivative.h"

void IIC_write_byte(byte data, byte adresa);
byte IIC_read_byte(byte adresa);
byte toBCD(byte decimalByte);
byte fromBCD(byte bcdByte);


// inicializace
void IIC_init(void){

  IIC1C_IICEN = 1;          // povolen IIC
  IIC1C_TXAK = 1;
  IIC1C_MST = 0;            // 
  IIC1F = 0x59;             // nastaven rychlosti na 100kHz
  IIC1S_SRW = 0;            // Master zapisuje
  IIC_write_byte(0x95,0x08);
  
}

// aktivace budk
void enableAlarm(byte budik){
  byte pom;
  if(budik <= 7)
    pom = IIC_read_byte(0x07) & 0xCF;
  
    IIC_write_byte(pom | (budik << 4), 0x07); 
}

// nastaven formtu asu
void setTimeFormat(byte format){
 byte time = IIC_read_byte(0x02);               // nateme pvodn hodnotu
 if(format == 0){                               // cas == 0 -> 24 hod format
      IIC_write_byte(time & 0x3F, 0x02);   
 }else if(format == 1){                      // cas == 1 -> 12 hod format
      IIC_write_byte(time | 0x40, 0x02);  
 }   
}


// zjitn formtu asu 12/24 hod
byte getTimeFormat(void){
  return ((IIC_read_byte(0x02) & 0x40) );             
} 


// povolen vstupn frekvence pes MFP pin
// !!! pokud je aktivn, nen mon pout peruen od budku !!! 
void outFrequency(byte povolit, byte frekvence){
 if(frekvence <= 7 && povolit <= 1){
   byte pom;
   povolit = povolit << 6;
   pom = IIC_read_byte(0x07) & 0x30;
   IIC_write_byte(povolit | pom | frekvence, 0x07); 
 }
  
}

//nastaven asu
void setTime(byte volba, byte hod){
  byte pom;
  byte hodnota = toBCD(hod);
  
  switch(volba){
    case 1: IIC_write_byte(hodnota, 0x03); break;         //den v tdnu
    case 2: IIC_write_byte(hodnota, 0x04); break;         //den v msc
    case 3: IIC_write_byte(hodnota, 0x05); break;         //msc
    case 4: IIC_write_byte(hodnota, 0x06); break;         //rok
    case 5: IIC_write_byte(hodnota | getTimeFormat(), 0x02); break;  //hodiny
    case 6: IIC_write_byte(hodnota, 0x01); break;         //minuty
    case 7: IIC_write_byte(hodnota | 0x80, 0x00); break;  //sekundy
    case 8: pom = (IIC_read_byte(0x02) & 0x5F);
            IIC_write_byte(pom | (hodnota << 5),0x02);    //AM/PM
  
    default: break;    
  }  
}
                 

//naten asu
byte getTime(byte cislo){
  
  byte data;
  
  switch(cislo){
    case 1: data = IIC_read_byte(0x03) & 0xDF; break;     //den v tdnu
    case 2: data = IIC_read_byte(0x04); break;            //den v msc
    case 3: data = IIC_read_byte(0x05) & 0x1F; break;            //msc
    case 4: data = IIC_read_byte(0x06); break;            //rok
    case 5: data = (getTimeFormat() == 0) ? IIC_read_byte(0x02) & 0x3F
                   : IIC_read_byte(0x02) & 0x1F; break;   //hodiny
    case 6: data = IIC_read_byte(0x01); break;            //minuty
    case 7: data = IIC_read_byte(0x00) & 0x7F; break;     //sekundy
    case 8: data = (IIC_read_byte(0x02) & 0x20) >> 5;     // AM/PM AM = 0, PM = 1
   
    default: break;    
  }
    
  return fromBCD(data); 
}

// zjitn jestli budk zvon
byte alarm(byte cislo){
    byte hodnota;
    if(cislo == 1)
      hodnota = IIC_read_byte(0x0D);
    
    if(cislo == 2)
      hodnota = IIC_read_byte(0x14);
  
    return ((hodnota & 0x08) >> 3);
}

//nastaven budk
void setAlarm(byte budik, byte cislo, byte hod){
  byte pom;
  byte hodnota = toBCD(hod);
  
  if(budik == 1){
  
    switch(cislo){ 
      case 1: pom = IIC_read_byte(0x0D) & 0x0F;
              IIC_write_byte(0x80 | (hod << 4) | pom,0x0D); break; // rezim
      case 2: pom = IIC_read_byte(0x0D) & 0x70;
              IIC_write_byte(0x80 | pom | hodnota ,0x0D); break; // den v tydnu
      case 3: IIC_write_byte(hodnota,0x0E); break; // den v mesici
      case 4: IIC_write_byte(hodnota,0x0F); break; // mesic
      case 5: IIC_write_byte(hodnota,0x0C); break;  // hodiny
      case 6: IIC_write_byte(hodnota,0x0B); break; // minuta
      case 7: IIC_write_byte(hodnota,0x0A); break; // sekunda 
      case 8: pom = IIC_read_byte(0x0C);
              IIC_write_byte(pom | (hod << 5),0x0C); break; // AM/PM
      default: break;        
    }
        
  }else if(budik == 2){
    
    switch(cislo){ 
      case 1: pom = IIC_read_byte(0x14) & 0x0F;
              IIC_write_byte(0x80 | (hod << 4) | pom,0x14); break; // rezim
      case 2: pom = IIC_read_byte(0x14) & 0x70;
              IIC_write_byte(0x80 | pom | hodnota ,0x14); break; // den v tydnu
      case 3: IIC_write_byte(hodnota,0x15); break; // den v mesici
      case 4: IIC_write_byte(hodnota,0x16); break; // mesic
      case 5: IIC_write_byte(hodnota,0x13); break;  // hodiny
      case 6: IIC_write_byte(hodnota,0x12); break; // minuta
      case 7: IIC_write_byte(hodnota,0x11); break; // sekunda 
      case 8: pom = IIC_read_byte(0x13);
              IIC_write_byte(pom | (hod << 5),0x13); break; // AM/PM
      default: break;        
    } 
  }  
}

// naten nastaven budk
byte getAlarm(byte budik, byte cislo){

  byte hodnota = 0;
  
  if(budik == 1){
  
    switch(cislo){ 
      case 1: hodnota = (IIC_read_byte(0x0D) & 0x70) >> 4; break; // rezim
      case 2: hodnota = IIC_read_byte(0x0D) & 0x07; break; // den v tydnu
      case 3: hodnota = IIC_read_byte(0x0E); break; // den v mesici
      case 4: hodnota = IIC_read_byte(0x0F); break; // mesic
      case 5: hodnota = (getTimeFormat() == 0) ? IIC_read_byte(0x0C) & 0x3F
                        : IIC_read_byte(0x0C) & 0x1F; break;   //hodiny
      case 6: hodnota = IIC_read_byte(0x0B); break; // minuta
      case 7: hodnota = IIC_read_byte(0x0A); break; // sekunda 
      case 8: hodnota = (IIC_read_byte(0x0C) & 0x20) >> 5; break; // AM/PM
      default: break;        
    }
        
  }else if(budik == 2){
    
    switch(cislo){ 
      case 1: hodnota = (IIC_read_byte(0x14) & 0x70) >> 4; break; // rezim
      case 2: hodnota = IIC_read_byte(0x14) & 0x07; break; // den v tydnu
      case 3: hodnota = IIC_read_byte(0x15); break; // den v mesici
      case 4: hodnota = IIC_read_byte(0x16); break; // mesic
      case 5: hodnota = (getTimeFormat() == 0) ? IIC_read_byte(0x13) & 0x3F
                        : IIC_read_byte(0x13) & 0x1F; break;   //hodiny
      case 6: hodnota = IIC_read_byte(0x12); break; // minuta
      case 7: hodnota = IIC_read_byte(0x11); break; // sekunda 
      case 8: hodnota = (IIC_read_byte(0x13) & 0x20) >> 5; break; // AM/PM
      default: break;        
    }     
  }
  return fromBCD(hodnota);  
}


// reset zvonn
void resAlarm(byte budik){
  if(budik == 1){
    
    byte pom = IIC_read_byte(0x0D);
    IIC_write_byte(pom & 0xF7,0x0D);
    
  }else if(budik == 2){

    byte pom = IIC_read_byte(0x14);
    IIC_write_byte(pom & 0xF7,0x14); 
  } 
}


 

// zjitn pestupnho roku
byte isLeapYear(void){
  return (IIC_read_byte(0x05) & 0x20) >> 5;       // 1 = je pestupn rok
  
}


// zapi Byte do voln pamti
void writeByte(byte hodnota, byte adresa){

  byte adr = adresa;
   
  if(adr >= 0x20 && adr <= 0x5F)
  {
     IIC_write_byte(hodnota, adr);
  }
 
}


// nati Byte z voln pamti
byte readByte(byte adresa){

  byte data = 0; 
  byte adr = adresa;
   
  if(adr >= 0x20 && adr <= 0x5F)
  {
     data = IIC_read_byte(adr);
  }
  
  return data;
}


//BCD pevod
byte toBCD(byte decimalByte){
  
  return (((decimalByte / 10) << 4) | (decimalByte % 10));
}

byte fromBCD(byte bcdByte){
  
  return ((((bcdByte & 0xF0) >> 4) * 10) + (bcdByte & 0x0F));
  
}

  
void IIC_write_byte(byte data, byte adresa){
 
  while(IIC1S_BUSY)__RESET_WATCHDOG();        // ekej dokud nebude IIC volna
 
  IIC1C_TXAK = 0;           // TXAK = 0;
  IIC1C |= 0x30;            // generovn START;
 
  IIC1D = 0xDE;             // adresa RTC - zpis
  while (!IIC1S_IICIF);     // wait until IBIF;
  IIC1S_IICIF=1;            // clear the interrupt event flag;
  while(IIC1S_RXAK);        // check for RXAK;
  
  IIC1D = adresa;           // poli adresu bytu
  while (!IIC1S_IICIF);     // wait until IBIF;
  IIC1S_IICIF=1;            // clear the interrupt event flag;
  while(IIC1S_RXAK);        // check for RXAK;

  IIC1D = data;
  while (!IIC1S_IICIF);     // wait until IBIF;
  IIC1S_IICIF=1;            // clear the interrupt event flag;
  while(IIC1S_RXAK);        // check for RXAK;
  __RESET_WATCHDOG()

  IIC1S_IICIF=1;            // clear the interrupt event flag;
  IIC1C_MST = 0;            // generate STOP condition;
}



byte IIC_read_byte(byte adresa){

  byte data;
   
  while(IIC1S_BUSY)__RESET_WATCHDOG();     // ekej dokud nebude IIC volna
  
  IIC1C_TXAK = 0;           // stejny jako u zpisu
  IIC1C |= 0x30;            
  IIC1D = 0xDE;             
  while (!IIC1S_IICIF);     
  IIC1S_IICIF=1;            
  while(IIC1S_RXAK);        
  
  IIC1D = adresa;             
  while (!IIC1S_IICIF);     
  IIC1S_IICIF=1;            
  while(IIC1S_RXAK);       
  
  IIC1C_RSTA = 1;           // opakuj START
  IIC1D = 0xDF;             // adresa RTC - ten
  while (!IIC1S_IICIF);     // wait until IBIF;
  IIC1S_IICIF=1;            // clear the interrupt event flag;
  while (IIC1S_RXAK);       // check for RXAK;
  
  IIC1C_TX = 0;             // reim receive;
  IIC1C_TXAK = 1;           // acknowledge disable;
  data = IIC1D;             // nati data
  while (!IIC1S_IICIF);     // wait until IBIF;
  __RESET_WATCHDOG()
  IIC1S_IICIF=1;            // clear the interrupt event flag;
  IIC1C_MST = 0;            // generate stop signal;
  data = IIC1D;             // read right data;
  return data;
}
